/*
 * XPSDriver.h
 * 
 * Asyn driver that inherits from the asynPortDriver class to control XPS analog and digital I/O
 *
 * Author: Mark Rivers
 *
 * Created January 25, 2015
 */

#include "asynPortDriver.h"

/* These are the drvInfo strings that are used to identify the parameters.
 * They are used by asyn clients, including standard asyn device support */
#define P_AnalogInputString      "ANALOG_INPUT"               /* asynFloat64,       r/o */
#define P_AnalogOutputString     "ANALOG_OUTPUT"              /* asynFloat64,       r/w */
#define P_DigitalInputString     "DIGITAL_INPUT"              /* asynUInt32Digital, r/o */
#define P_DigitalOutputString    "DIGITAL_OUTPUT"             /* asynUInt32Digital, r/w */
#define P_FirmwareVersionString  "FIRMWARE_VERSION"           /* asynOctet,         r/o */

#define MAX_MESSAGE_SIZE 80

/** Class to control the XPS analog and digital I/O */
class XPSDriver : public asynPortDriver {

public:
    XPSDriver(const char *portName, const char *BPMServerName, double pollDelay);
    
    /* These are the methods that we override from asynPortDriver */
    asynStatus writeUInt32Digital(asynUser *pasynUser, epicsUInt32 mask, epicsUInt32 value);
    asynStatus writeFloat64(asynUser *pasynUser, epicsFloat64 value);
    void report(FILE *fp, int details);
    // This should be private, but is called from C so must be public
    void pollerThread(void);

private:
    /** Values used for pasynUser->reason, and indexes into the parameter library. */
    int P_AnalogInput;
    #define FIRST_XPS_COMMAND P_AnalogInput
    int P_AnalogOutput;
    int P_DigitalInput;
    int P_DigitalOutput;
		int P_FirmwareVersion;
    #define LAST_XPS_COMMAND P_FirmwareVersion

    /* These are the methods that are new to this class */
  	asynStatus writeReadXPS();
		
    /* Our data */
    asynUser *pasynUserXPS_;
    char *XPSPortName_;
    char outString_[MAX_MESSAGE_SIZE];
    char inString_[MAX_MESSAGE_SIZE];
    double pollDelay_;
};

#define NUM_XPS_PARAMS (&LAST_XPS_COMMAND - &FIRST_XPS_COMMAND + 1)
