/*
 * XPSTemplate.cpp
 * Asyn driver that controls the analog and digital I/O on a Newport XPS 
 *
 * Author: Mark Rivers
 *
 * Created January 25, 2015
 */

#include <string.h>
#include <stdlib.h>

#include <epicsThread.h>
#include <epicsString.h>
#include <asynOctetSyncIO.h>
#include <iocsh.h>

#include <epicsExport.h>

#include "XPSDriver.h"

#define XPS_TIMEOUT 1.0
#define MAX_ANALOG_INPUTS   4
#define MAX_ANALOG_OUTPUTS  4
#define MAX_ADDRESS MAX_ANALOG_INPUTS

static const char *analogInputNames[MAX_ANALOG_INPUTS] = {
    "GPIO2.ADC1", /* Analog Input # 1 of the I/O board connector # 2 */
    "GPIO2.ADC2", /* Analog Input # 2 of the I/O board connector # 2 */
    "GPIO2.ADC3", /* Analog Input # 3 of the I/O board connector # 2 */
    "GPIO2.ADC4", /* Analog Input # 4 of the I/O board connector # 2 */
};
static const char *analogOutputNames[MAX_ANALOG_OUTPUTS] = {
    "GPIO2.DAC1", /* Analog Output # 1 of the I/O board connector # 2 */
    "GPIO2.DAC2", /* Analog Output # 2 of the I/O board connector # 2 */
    "GPIO2.DAC3", /* Analog Output # 3 of the I/O board connector # 2 */
    "GPIO2.DAC4", /* Analog Output # 4 of the I/O board connector # 2 */
};

static const char *digitalInputName  = "GPIO3.DI";
static const char *digitalOutputName = "GPIO3.DO";

static const char *driverName="XPSDriver";
static void pollerThread(void *drvPvt);

/** Constructor for the XPSDriver class.
  * Calls constructor for the asynPortDriver base class.
  * \param[in] portName The name of the asyn port driver to be created.
  * \param[in] XPSName The asyn port name of the drvAsynIPPort connect to the XPS
  */
XPSDriver::XPSDriver(const char *portName, const char *XPSPortName, double pollDelay) 
   : asynPortDriver(portName, 
                    MAX_ADDRESS,   /* maxAddr */ 
                    NUM_XPS_PARAMS,
                    asynFloat64Mask | asynUInt32DigitalMask | asynOctetMask | asynDrvUserMask, /* Interface mask */
                    asynFloat64Mask | asynUInt32DigitalMask | asynOctetMask,                   /* Interrupt mask */
                    ASYN_CANBLOCK | ASYN_MULTIDEVICE, /* asynFlags.  This driver blocks it is multi-device */
                    1, /* Autoconnect */
                    0, /* Default priority */
                    0) /* Default stack size*/    
{
    asynStatus status;
    const char *functionName = "XPSDriver";
    
    // Create the parameters for this driver
    
    // Copy the XPSPortName and pollDelay to private member data

    // Connect to the server
        
    // Read the firmware version
    
    // Create the thread that polls the XPS
    
}

/** Called when asyn clients call pasynUInt32Digital->write().
  * \param[in] pasynUser pasynUser structure that encodes the reason and address.
    * \param[in] mask Mask value for write operation
  * \param[in] value Value to write. */
asynStatus XPSDriver::writeUInt32Digital(asynUser *pasynUser, epicsUInt32 value, epicsUInt32 mask)
{
    int function = pasynUser->reason;
    int status = asynSuccess;
    int channel;
    const char *paramName;
    const char* functionName = "writeUInt32Digital";

    // Get the address of this channel
    
    // Set the parameter in the parameter library.
    
    // Fetch the parameter string name for possible use in debugging

    // If the function if P_DigitalOutput then write to the hardware
    if (function == P_DigitalOutput) {

    } 
    else {
        /* All other parameters just get set in parameter list, no need to
         * act on them here */
    }
    
    // Do callbacks so higher layers see any changes   

    // Print error messages and success messages     
    if (status) {
    
    }
    else {       
    
    }

    return (asynStatus)status;
}

/** Called when asyn clients call pasynFloat64->write().
  * This function sends a signal to the simTask thread if the value of P_UpdateTime has changed.
  * For all  parameters it  sets the value in the parameter library and calls any registered callbacks.
  * \param[in] pasynUser pasynUser structure that encodes the reason and address.
  * \param[in] value Value to write. */
asynStatus XPSDriver::writeFloat64(asynUser *pasynUser, epicsFloat64 value)
{
    int function = pasynUser->reason;
    int status = asynSuccess;
    int channel;
    const char *paramName;
    const char* functionName = "writeFloat64";

    // Get the address of this channel
    
    // Set the parameter in the parameter library.
    
    // Fetch the parameter string name for possible use in debugging
    
    // If the function if P_AnalogOutput then write to the hardware
    if (function == P_AnalogOutput) {

    } else {
        /* All other parameters just get set in parameter list, no need to
         * act on them here */
    }
    
    // Do callbacks so higher layers see any changes   

    // Print error messages and success messages     
    if (status) {

    }
    else {
    
    } 

    return (asynStatus)status;
}

/** Writes a string to the XPS and reads the response. */
asynStatus XPSDriver::writeReadXPS()
{
    size_t nread;
    size_t nwrite;
    asynStatus status;
    bool replyOK;
    char *eom;
    int eomReason;
    int len;
    char temp[MAX_MESSAGE_SIZE];
    const char *functionName="writeReadXPS";


    // Send the string to the XPS and read the response with pasynOctetSyncIO->writeRead() into temp
      
    // Make sure the response begins with the characters "0,"
    
    // Make sure the response ends with the string ",EndOfAPI"
    
    // Copy string after "0," up to ",EndOfAPI" into inString_
    
    // Print ASYN_TRACEIO_DRIVER message about strings sent and received
    
  return status;
}


static void pollerThread(void *drvPvt)
{
    XPSDriver *pPvt = (XPSDriver *)drvPvt;
    
    pPvt->pollerThread();
}

/** Read thread to read the data from the electrometer when it is in continuous acquire mode.
  * Reads the data and does callbacks.
  */
void XPSDriver::pollerThread(void)
{
    int status;
    int i;
    double aiValue;
    epicsUInt32 uiValue;
    //static const char *functionName = "readThread";

    /* Loop forever */
    lock();
    while (1) {
        // Read each of the analog inputs
        for (i=0; i<MAX_ANALOG_INPUTS; i++) {
            // Create string to send to XPS
            
            // Do I/O to XPS
            
            // If response is OK convert response to double, set value in parameter library and do callbacks
        }
        // Read the digital input register
        // Create string to send to XPS

        // Do I/O to XPS

        // If response is OK convert response to epicsUInt32, set value in parameter library and do callbacks

        // Unlock the port, sleep for pollDelay_, and lock the port
    }
}

/** Report  parameters 
  * \param[in] fp The file pointer to write to
  * \param[in] details The level of detail requested
  */
void XPSDriver::report(FILE *fp, int details)
{
    char firmwareVersion[MAX_MESSAGE_SIZE];
        

    // Get the firmware version from the parameter library and print information about the driver
    
    // Call the base class report method
}


/* Configuration routine.  Called directly, or from the iocsh function below */

extern "C" {

/** EPICS iocsh callable function to call constructor for the XPSDriver class.
  * \param[in] portName The name of the asyn port driver to be created.
  * \param[in] XPSPortName The name of the asyn communication port to the server
  *            created with drvAsynIPPortConfigure */
int XPSDriverConfigure(const char *portName, const char *XPSPortName, double pollDelay)
{
    new XPSDriver(portName, XPSPortName, pollDelay);
    return asynSuccess;
}


/* EPICS iocsh shell commands */

static const iocshArg initArg0 = { "portName",iocshArgString};
static const iocshArg initArg1 = { "XPSPortName",iocshArgString};
static const iocshArg initArg2 = { "XPSPortName",iocshArgDouble};
static const iocshArg * const initArgs[] = {&initArg0,
                                            &initArg1,
                                            &initArg2};
static const iocshFuncDef initFuncDef = {"XPSDriverConfigure",3,initArgs};
static void initCallFunc(const iocshArgBuf *args)
{
    XPSDriverConfigure(args[0].sval, args[1].sval, args[2].dval);
}

void XPSDriverRegister(void)
{
    iocshRegister(&initFuncDef,initCallFunc);
}

epicsExportRegistrar(XPSDriverRegister);

}  /* extern "C" */

